==> jami-20250917.0.tar.gz: jami-20250917.0/daemon/src/map_utils.h <==
/*
 *  Copyright (C) 2004-2025 Savoir-faire Linux Inc.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <https://www.gnu.org/licenses/>.
 */
#pragma once

#include <vector>
#include <iterator>
#include <algorithm>
#include <tuple>

namespace jami {
namespace map_utils {

///< Return the N-th type of a tuple type used as the Container compliant value type
template<typename C, std::size_t N>
using type_element =
    typename std::remove_cv<typename std::tuple_element<N, typename C::value_type>::type>::type;

///< Extract in a std::vector object each N-th values of tuples contained in a Container compliant
///< object \a container.
template<std::size_t N, typename C>
inline std::vector<type_element<C, N>>
extractElements(const C& container)
{
    std::vector<type_element<C, N>> result;
    if (container.size() > 0) {
        result.resize(container.size());
        auto iter = std::begin(container);
        std::generate(std::begin(result), std::end(result), [&] { return std::get<N>(*iter++); });
    }
    return result;
}

template<typename M>
inline auto
extractKeys(const M& map) -> decltype(extractElements<0>(map))
{
    return extractElements<0>(map);
}

template<typename M>
inline auto
extractValues(const M& map) -> decltype(extractElements<1>(map))
{
    return extractElements<1>(map);
}

} // namespace map_utils
} // namespace jami

==> jami-20251212.0.tar.gz: jami-20251212.0/daemon/src/map_utils.h <==
/*
 *  Copyright (C) 2004-2025 Savoir-faire Linux Inc.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <https://www.gnu.org/licenses/>.
 */
#pragma once

#include <vector>
#include <iterator>
#include <algorithm>
#include <tuple>

namespace jami {
namespace map_utils {

///< Return the N-th type of a tuple type used as the Container compliant value type
template<typename C, std::size_t N>
using type_element = typename std::remove_cv<typename std::tuple_element<N, typename C::value_type>::type>::type;

///< Extract in a std::vector object each N-th values of tuples contained in a Container compliant
///< object \a container.
template<std::size_t N, typename C>
inline std::vector<type_element<C, N>>
extractElements(const C& container)
{
    std::vector<type_element<C, N>> result;
    if (container.size() > 0) {
        result.resize(container.size());
        auto iter = std::begin(container);
        std::generate(std::begin(result), std::end(result), [&] { return std::get<N>(*iter++); });
    }
    return result;
}

template<typename M>
inline auto
extractKeys(const M& map) -> decltype(extractElements<0>(map))
{
    return extractElements<0>(map);
}

template<typename M>
inline auto
extractValues(const M& map) -> decltype(extractElements<1>(map))
{
    return extractElements<1>(map);
}

} // namespace map_utils
} // namespace jami
